#!/bin/bash
#
#$ -cwd
#$ -V
#$ -j y
#$ -w e
#$ -S /bin/bash
#$ -l h_core=0
#$ -N {{jobname}}
#$ -q ibgpu.q
#$ -o {{logfile}}

date
# export NCCL_DEBUG=INFO
# export NCCL_DEBUG_SUBSYS=ALL
{{run_commands}}
date