#!/bin/bash --login
source ~/.bashrc
{{env_init_commands}}
nvidia-smi
nvidia-smi topo -m
python3 {{runner}} {{version}} {{a_index}} {{t_index}} {{r_index}} {{configfile}} {{pylogfile}}
{{env_end_commands}}